#include <windows.h>

#ifndef __DATASTR_H__
#define __DATASTR_H__

#define MAX_DATA 4096
#define MAX_FNAME 64
#define MAX_FIELD 256

typedef struct _PDF_PARAM{
	BOOL bErrorLevelAuto;
	UINT uErrLev;
	UINT wRows;
	UINT wCols;
	BOOL bAutoSize;
	BOOL bFixRowsorCol;
	BOOL bFixRows;
	BOOL bFixCols;
}PDF_PARAM;

typedef struct _BARCODE{
	UINT uType;				//Barcode type 
	CHAR szData[MAX_DATA];	//Barcode data 
	double uHeightMM;			//Barcode height in mm
	double uWidthMM;			//Barcode width in mm (usable only for PDF 417)
	double uHeightSize;		//koeff for height (1.000, 0.750, ...) 
	double uModulBreite;	//Modulbreite (in m)
	UINT uAddon;			//Addon (0;2;5)
	UINT uScalProc;			//for EAN in %( 82, 90 ,100, ...)
	int	 iReduc;		    //reduction of barcode bars	in % of narrow width (5,10,15,...)
	BOOL bPlainText;		//if TRUE text is necessary for EAN type barcodes
	int  iCheckSumm;		//returns a checksumm for barcode 
	BOOL bCheckSum;			//is necessary to calculate checksum
	CHAR chStartCodabar;	//start char for CODABAR (A,B,C,D)
	CHAR chStopCodabar;		//stop char for CODABAR (A,B,C,D)
	double dNarrowToWide;	//Narrow to wide relation (2.0, 2.1,..., 3.0 )
	BOOL bLeitcode;
	UINT numCustChars;		//number of chars allowed for "Kundenerkennung"
	double uSkalierung;		//for UPC EAN 128 (0.25, 0.30, ... 1.20 ) 
	BOOL bOnlyTTFont;		//only TT font is possible to use for plaintext
	CHAR szFontName[MAX_FNAME]; //face name for plaintext font
	UINT uFontSize;
	COLORREF clrBack;
	COLORREF clrFore;
	UINT uTextAlignment;
	BOOL bHIBC;
	//params for PDF417 type
	BOOL bErrorLevelAuto;	// auto error level for PDF417
	UINT uErrLev;			// error level for PDF417 (auto err level FALSE) (0,1,...8)
	UINT uPDFSize;			// 0 - auto height and breite ( max height and min width)
							// 1 - height is fixed and width will be calculated
							// 2 -  width is fixed and height will be calculated

	// Codablock F ------------------------------------------
	UINT uRows; //    - 0  - Auto
				//    - 44 - max value

	// DataMatrix  ------------------------------------------
	UINT uEncode;	//    0 - Auto
					//	  1 - ASCII
					//	  2 - C40
					//	  3 - TEXT
					//	  4 - BASE256

	UINT uMatrix;	//	0 - Auto
					//	other values 	 
	BOOL bTilde;

	// MaxiCode    ------------------------------------------
	CHAR szService[MAX_FIELD];
	CHAR szPostalCode[MAX_FIELD];
	CHAR szCountry[MAX_FIELD];
	UINT uSymb;			// 		(1-x)
	UINT uSymbNumber;	//		(1-x)
	UINT uMode;			//		(2-6) 

	// Aztec	   ------------------------------------------
	UINT uLevel;
	UINT uLayers;
	BOOL bESC;

	// Demo
	BOOL bDemo;

	UINT uAngle;
	BOOL bRuheZone;
	int iOutlineWidth;
	DWORD* pData;

}BARCODE;

#endif


/* 
uType		Name			MinData	 MaxData (in szData)

0		"EAN 13"		12		12			
1		"EAN 8"			7		 7	
2		"EAN-Velocity"		1		 6
3		"UPC A"			11		11
4		"UPC E"			6		 6
5		"Code 128",		1		40
6		"Code 128 B"		1		40
7		"Code 128 C"		2		60
8		"Code EAN 128"		1		48
9		"Code UPS 128"		15		15
10		"Code 39"		1		30
11		"Code 39 extended"	1		30
12		"PZN"			6		 6
13		"Code 93"		1		30
14		"Code 93 extended"	1		30
15		"2/5 Industrie"		1		30
16		"2/5 Interleaved"	1		60
17		"2/5 Post"		11		11
18		"Codabar"		1		40
19		"PDF417"		1		1850
20		"MSI_Plessey"		1		100
21		"Codablock F"		8		2725
22		"Data Matrix"		1		2700
23		"MaxiCode"		1		93
24		"Aztec"			1		2869
25		"QRCode"		1		2953
26		"GS1 Databar"		1		14
27		"EAN-UCC Composite"	12		12
	
*/




// exported functions from vbar32.dll //////////////////////

// initalizing of DLL and barcode structures 
extern "C" BOOL InitBarcode( void );	
	
// cleaning of DLL and barcode structures 
extern "C" BOOL ExitBarcode( void );	    

// input  - pointer to BARCODE structure with barcode parameters
// output - pointer to METAFILEPICT structure (global handle) with barcode metafile
extern "C" HGLOBAL GetBarcode( BARCODE* );


// input  - pointer to BARCODE structure with barcode parameters
// output - pointer to HENHMETAFILE handle for enhanced barcode metafile
extern "C" HENHMETAFILE GetBarcodeEnhMetafile( BARCODE* );


// input  - pointer to BARCODE structure with barcode parameters
//	  - path of exported bitmap file (LPCTSTR)
//	  - resolution of bitmap (UINT)
//	  - colors per pixel of bitmap (UINT)
// output - TRUE if export was successful, otherwise FALSE
extern "C" BOOL SaveBarcodeAsBMP( BARCODE*, LPCTSTR, UINT, UINT );

// input  - pointer to BARCODE structure with barcode parameters
//	  - path of exported metafile file (LPCTSTR)
// output - TRUE if export was successful, otherwise FALSE
extern "C" BOOL SaveBarcodeAsWMF( BARCODE*, LPCTSTR );






/* Matrix sizes for DataMatrix


1 = 10 x 10
2 = 14 x 14
3 = 16 x 16
4 = 18 x 18
5 = 20 x 20
6 = 22 x 22
7 = 24 x 24
8 = 26 x 26
9 = 32 x 32
10= 36 x 36
11= 40 x 40
12= 44 x 44
13= 48 x 48
14= 52 x 52
15= 64 x 64
16= 72 x 72
17= 80 x 80
18= 88 x 88
19= 96 x 96
20= 104 x 104
21= 120 x 120
22= 132 x 132
23= 144 x 144
24= 8 x 18
25= 8 x 32
26= 12 x 26
27= 12 x 36
28= 16 x 36
29= 16 x 48

*/

