/*--------------------------------------------------------
   HELLOWIN.C - Displays "Hello, Windows!" in client area
                (c) Charles Petzold, 1992
  --------------------------------------------------------*/

#include <windows.h>
#include "resource.h"
#include "datastr.h"

#define HIMETRIC_INCH   2540    // HIMETRIC units per inch


long WINAPI WndProc (HWND, UINT, UINT, LONG) ;

HMETAFILE hMfBarcode = NULL;
BARCODE	 pBarcode;

long	  xExtBarcode;
long	  yExtBarcode;
void	  SetBarcodeParams( BARCODE* );

void CopyToClipboard( HWND hWnd, HMETAFILE hMetaFile );
void CopyEnhToClipboard( HWND hWnd, HMETAFILE hMetaFile );
void PrintBarcode( HWND hwnd, HMETAFILE hMfBarcode);

void DrawBarcode( HDC hdc, HMETAFILE hMetafile, int xSize, int ySize );
int CALLBACK EnumMFProc(HDC hdc, HANDLETABLE FAR *lpht, METARECORD  FAR *lpmr, int cObj, LPARAM lParam);

HENHMETAFILE GetBarcodeEnhMetafile(BARCODE* barc);

void GetMetafile(){

	 METAFILEPICT* pMfBarcPict = (METAFILEPICT*)GetBarcode( (BARCODE*) &pBarcode);
	 if( pMfBarcPict != NULL ) {
		hMfBarcode  = pMfBarcPict->hMF;
		xExtBarcode = pMfBarcPict->xExt;
		yExtBarcode = pMfBarcPict->yExt;
	 }
}


int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
                    LPSTR lpszCmdParam, int nCmdShow)
     {
     static char szAppName[] = "HelloWin" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASS    wndclass ;

     if (!hPrevInstance)
	 {													  
          wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
          wndclass.lpfnWndProc   = WndProc ;
          wndclass.cbClsExtra    = 0 ;
          wndclass.cbWndExtra    = 0 ;
          wndclass.hInstance     = hInstance ;
          wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
          wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
          wndclass.hbrBackground = (HBRUSH)GetStockObject (WHITE_BRUSH) ;
          wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU);//NULL ;
          wndclass.lpszClassName = szAppName ;

          RegisterClass (&wndclass) ;
	 }				  

     hwnd = CreateWindow (szAppName,       // window class name
                    "Barcode DLL sample",   // window caption
                    WS_OVERLAPPEDWINDOW,   // window style
                    CW_USEDEFAULT,         // initial x position
                    CW_USEDEFAULT,         // initial y position
                    CW_USEDEFAULT,         // initial x size
                    CW_USEDEFAULT,         // initial y size
                    NULL,                  // parent window handle
                    NULL,                  // window menu handle
                    hInstance,             // program instance handle
                    NULL) ;                // creation parameters

	 ////////// The barcode stuff ///////////////////////////////
	 ///////////////////////////////////////////////////////////
	InitBarcode();
	SetBarcodeParams((BARCODE*) &pBarcode);
	GetMetafile();  // get metafile handle for onscreen drawing
	 ///////////////////////////////////////////////////////////
		
     ShowWindow (hwnd, nCmdShow) ;
     UpdateWindow (hwnd) ;
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }


void DrawBarcode( HDC hdc, HMETAFILE hMetafile, int xSize, int ySize ){

	if( hMetafile == NULL ) GetMetafile();

	if( hMetafile ){

		SIZE		 Size;
		POINT		 point;
		int cxPerInch = GetDeviceCaps(hdc,LOGPIXELSX);
		int cyPerInch = GetDeviceCaps(hdc,LOGPIXELSY);

		POINT p;
		p.x = MulDiv(xSize, cxPerInch, HIMETRIC_INCH);
		p.y = MulDiv(ySize, cyPerInch, HIMETRIC_INCH);
																	  
		SetMapMode(hdc, MM_ANISOTROPIC);
		SetWindowExtEx(hdc, xSize, ySize, &Size);
		SetViewportOrgEx(hdc, 0, 0, &point);
		// set true barcode size
		SetViewportExtEx(hdc, p.x, p.y, &Size);   //
		PlayMetaFile( hdc, (HMETAFILE)hMetafile);
	}
}



long WINAPI WndProc (HWND hwnd, UINT message,UINT wParam, LONG lParam)
{

     HDC         hdc ;
     PAINTSTRUCT ps ;

     switch (message) {

		case WM_COMMAND:
			switch(LOWORD(wParam)){

			case IDM_SAVE_WMF:
				InitBarcode();
				if( !SaveBarcodeAsWMF( (BARCODE*) &pBarcode, "c:\\test.wmf" ) ) {
					MessageBox( hwnd, "Impossible to save barcode as WMF file!", "Error", MB_OK );
				}
				else {
					MessageBox(hwnd, "Metafile was saved to c:\\test.bmp!", "Info", MB_OK);
				}

				ExitBarcode();
				hMfBarcode = NULL;
				break;
				
			case IDM_SAVE_BMP:

				 InitBarcode();
				 SetBarcodeParams((BARCODE*) &pBarcode);
				 if( !SaveBarcodeAsBMP( (BARCODE*) &pBarcode, "c:\\test.bmp", 150, 8 ) ) {
					
					 MessageBox( hwnd, "Impossible to save barcode as bitmap file!", "Error", MB_OK );
				 }
				 else {
					 MessageBox(hwnd, "Bitmap was saved to c:\\test.bmp!", "Info", MB_OK);
				 }
				 ExitBarcode();
				 hMfBarcode = NULL;
				 break;


 			case ID_SAVE_EMF:
				{
				 InitBarcode();
				 SetBarcodeParams((BARCODE*) &pBarcode);
				 HENHMETAFILE hEM = GetBarcodeEnhMetafile((BARCODE*) &pBarcode);
				 if (hEM != NULL) {
					 CopyEnhMetaFile(hEM, "c:\\test.emf");
					 MessageBox(hwnd, "Enhanced metafile was saved to c:\\test.emf!", "Info", MB_OK);
				 }
				 else					 
					 MessageBox( hwnd, "Enhanced metafile problems!", "Error", MB_OK );
				 ExitBarcode();
				 hMfBarcode = NULL;
				}
				 break;


			case ID_COPY_CLIPBOARD:
				{
				//CopyEnhToClipboard(hwnd,hMfBarcode);
				CopyToClipboard(hwnd,hMfBarcode);
				}
				break;
			
			case ID_PRINT:
				PrintBarcode(hwnd,hMfBarcode);
				break;

			case ID_EXIT:
				PostQuitMessage (0) ;
				break;
				
			}
			break;

		case WM_PAINT :
			{
			hdc = BeginPaint (hwnd, &ps) ;
			DrawBarcode(hdc, hMfBarcode, xExtBarcode, yExtBarcode);
			EndPaint (hwnd, &ps) ;
			}
			return 0 ;

		case WM_DESTROY :
			PostQuitMessage (0) ;
			return 0 ;
	}
    return DefWindowProc (hwnd, message, wParam, lParam) ;
}

void SetBarcodeParams( BARCODE* pBarcode){

	ZeroMemory( (BARCODE*) pBarcode, sizeof(BARCODE));

	//Fill BARCODE structure for EAN 13
	pBarcode->uType = 0;
	strcpy( pBarcode->szData, "123456789012" );
	pBarcode->uHeightMM = 0;
	pBarcode->uHeightSize = 1.000;
	pBarcode->uScalProc = 100;
	pBarcode->uAddon = 0;
	pBarcode->bPlainText = TRUE;
	pBarcode->clrFore = RGB(0,0,0);
	pBarcode->clrBack = RGB(255,255,0);
//	strcpy( pBarcode->szFontName, "Courier New" );  // for this type of barcode font OCR-B
													// will be used (if installed

/*
	//Fill BARCODE structure for CODE 128
	pBarcode->uType = 5;
	strcpy( pBarcode->szData, "123456" );
	pBarcode->uHeightMM = 25;
	pBarcode->uModulBreite = 330.0;
	pBarcode->clrFore = RGB(0,0,0);
	pBarcode->clrBack = RGB(255,255,0);
	pBarcode->bPlainText = TRUE;
	pBarcode->uModulBreite = 500;
	strcpy( pBarcode->szFontName, "Courier New" );
*/

/*
	//Fill BARCODE structure for PDF417
// 
	pBarcode->uType = 19;
	strcpy( pBarcode->szData, "2311231123sdfdsdsf12132213211");
	pBarcode->uHeightMM = 9;
	pBarcode->uWidthMM = 40;
	pBarcode->bPlainText = TRUE;
	pBarcode->clrFore = RGB(0,0,0);
	pBarcode->clrBack = RGB(255,255,255);
	pBarcode->uModulBreite = 900;
//
	pBarcode->bErrorLevelAuto = FALSE;
	pBarcode->uErrLev = 5;
	pBarcode->uPDFSize = 1;
	pBarcode->bDemo = FALSE;
*/
}

////////////////////////////////////////////////////////////////////
// Clipboard stuff

void CopyToClipboard( HWND hWnd, HMETAFILE hMetaFile ){

	HANDLE         hMem;
	LPMETAFILEPICT lpMFP;

	// Allocate memory for the METAFILEPICT struct
	hMem = GlobalAlloc(GMEM_ZEROINIT | GMEM_MOVEABLE | GMEM_DDESHARE, sizeof(METAFILEPICT));

	lpMFP = (LPMETAFILEPICT)GlobalLock(hMem);

	lpMFP->mm = MM_ANISOTROPIC;
	lpMFP->xExt = xExtBarcode;
	lpMFP->yExt = yExtBarcode;

	// Fill in the handle to the metafile.
	// Make a copy of the metafile because the memory block allocated
	// for it will be owned by the clipboard. The memory allocated for the
	// METAFILEPICT struct will also be owned by the clipboard.
	lpMFP->hMF = CopyMetaFile(hMetaFile, NULL);
	GlobalUnlock(hMem);

	// Copy the metafile to the clipboard.
	if( !OpenClipboard(hWnd) ) return;
	EmptyClipboard();
	SetClipboardData(CF_METAFILEPICT, hMem);
	CloseClipboard();

}


HENHMETAFILE  GetEnhMetafileHandle(METAFILEPICT* mp)
{

    HENHMETAFILE    		hTemp;
	DWORD                  dwSize;
	LPBYTE                 pBits;
	HDC                    hDC;
     
	if( (dwSize = GetMetaFileBitsEx( mp->hMF, 0, NULL )) == 0 ) {

		MessageBox( GetActiveWindow(), "Failed to Get MetaFile Bits Size", "Error Reading MetaFile", MB_OK );
		return NULL;
	}

	// Allocate that much memory
	if( (pBits =(LPBYTE) malloc( dwSize )) == NULL ) {

		MessageBox( GetActiveWindow(), "Failed to Allocate Memory for Metafile Bits", "Error Reading MetaFile", MB_OK );
		return NULL;
	}

	// Get the metafile bits
	if( GetMetaFileBitsEx( mp->hMF, dwSize, pBits ) == 0 ) {

		  free( pBits );
		  MessageBox( GetActiveWindow(), "Failed to Get MetaFile Bits","Error Reading MetaFile", MB_OK );
		  return NULL;
	}

	// Get a reference DC
	hDC = GetDC( NULL );

	// Make an enhanced metafile from the windows metafile
	hTemp = SetWinMetaFileBits( dwSize, pBits, hDC, mp );

	// Clean up
	ReleaseDC( NULL, hDC );
	free( pBits );

	if( hTemp == NULL ) MessageBox( GetActiveWindow(), "Failed to Create MetaFile from Bits", "Error Reading MetaFile", MB_OK );
	return hTemp;
}

void CopyEnhToClipboard( HWND hWnd, HMETAFILE hMetaFile ){

	HANDLE         hMem;
	LPMETAFILEPICT lpMFP;

	// Allocate memory for the METAFILEPICT struct
	hMem = GlobalAlloc(GMEM_ZEROINIT | GMEM_MOVEABLE | GMEM_DDESHARE, sizeof(METAFILEPICT));

	lpMFP = (LPMETAFILEPICT)GlobalLock(hMem);

	lpMFP->mm = MM_ANISOTROPIC;
	lpMFP->xExt = xExtBarcode;
	lpMFP->yExt = yExtBarcode;

	// Fill in the handle to the metafile.
	// Make a copy of the metafile because the memory block allocated
	// for it will be owned by the clipboard. The memory allocated for the
	// METAFILEPICT struct will also be owned by the clipboard.
	lpMFP->hMF = CopyMetaFile(hMetaFile, NULL);
	GlobalUnlock(hMem);

	HENHMETAFILE hEnhMem = GetEnhMetafileHandle(lpMFP);

	GlobalFree(hMem);

	// Copy the metafile to the clipboard.
	if( !OpenClipboard(hWnd) ) return;
	EmptyClipboard();
	SetClipboardData(CF_ENHMETAFILE, hEnhMem);
	CloseClipboard();

}



////////////////////////////////////////////////////////////////
// Printing stuff


// The Abort Procudure 
BOOL CALLBACK AbortProc( HDC hDC, int Error )
{
   MSG   msg;
   while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
   {
       TranslateMessage( &msg );
       DispatchMessage( &msg );
   }
   return TRUE;
}


// Initialize DOCINFO structure
void InitDocStruct( DOCINFO* di, char* docname) {

	// Always zero it before using it.
	memset( di, 0, sizeof( DOCINFO ) );
	// Fill in the required members.
	di->cbSize = sizeof( DOCINFO );
	di->lpszDocName = docname;
}

// Obtain printer device context
HDC GetPrinterDC(void) {

	PRINTDLG pdlg;

	// Initialize the PRINTDLG structure.
	memset( &pdlg, 0, sizeof( PRINTDLG ) );
	pdlg.lStructSize = sizeof( PRINTDLG );
	// Set the flag to return printer DC.
	pdlg.Flags = PD_RETURNDEFAULT | PD_RETURNDC;

	// Invoke the printer dialog box.
	PrintDlg( &pdlg );
	// hDC member of the PRINTDLG structure contains
	// the printer DC.
	return pdlg.hDC;
}


void PrintBarcode( HWND hwnd, HMETAFILE hMfBarcode) {

	HDC        hDC;
	DOCINFO    di;

	// Need a printer DC to print to.
	hDC = GetPrinterDC();

	// Did you get a good DC?
	if( !hDC) {
	   MessageBox(NULL, "Error creating DC", "Error", MB_APPLMODAL | MB_OK );
	   return;
	}

	// You always have to use an AbortProc().
	if( SetAbortProc( hDC, AbortProc ) == SP_ERROR ) {
	   MessageBox( NULL, "Error setting up AbortProc", "Error", MB_APPLMODAL | MB_OK);
	   return;
	}

	// Init the DOCINFO and start the document.
	InitDocStruct( &di, "Barcode");
	StartDoc( hDC, &di );

	// Print one page.
	StartPage( hDC );
	DrawBarcode(hDC, hMfBarcode, xExtBarcode, yExtBarcode);
	EndPage( hDC );

	// Indicate end of document.
	EndDoc( hDC );

	// Clean up
	DeleteDC( hDC );
}

 